///////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) typedef int GmbH and contributors.
//  All rights reserved. Licensed under the MIT open-source license.
//
///////////////////////////////////////////////////////////////////////////////

include "wamp.fbs";

namespace wamp;


/// WAMP key exchange interface. A key exchange facilitates secure encryption
/// key exchanges between WAMP service providers and consumers in a private
/// and non-custodiary way.
rpc_service IWampExchange(type: "interface",
                          uuid: "d3254a5d-1307-4f13-9976-94617498db44",
                          wampuri: "wamp")
{
    fixme (Void): Void;
}

rpc_service IWampProvider(type: "interface",
                          uuid: "e751ae3d-33ee-4f89-ad43-fa82db925274",
                          wampuri: "wamp")
{
    fixme (Void): Void;
}

rpc_service IWampConsumer(type: "interface",
                          uuid: "a1a314e9-7a5c-485c-87e7-b1bd4e1b1e0c",
                          wampuri: "wamp")
{
    fixme (Void): Void;
}
